<?php

namespace Drupal\openquestions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\openquestions\Model\OqApplicationInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the oq application entity class.
 *
 * @ContentEntityType(
 *	 id = "oq_application",
 *	 label = @Translation("OQ Application"),
 *	 label_collection = @Translation("OQ Applications"),
 *	 label_singular = @Translation("oq application"),
 *	 label_plural = @Translation("oq applications"),
 *	 label_count = @PluralTranslation(
 *		 singular = "@count oq applications",
 *		 plural = "@count oq applications",
 *	 ),
 *	 handlers = {
 *		 "view_builder" = "Drupal\openquestions\Display\OqApplicationViewBuilder",
 *		 "list_builder" = "Drupal\openquestions\List\OqApplicationListBuilder",
 *		 "views_data" = "Drupal\views\EntityViewsData",
 *		 "access" = "Drupal\openquestions\Access\OqApplicationAccessControlHandler",
 *		 "form" = {
 *			 "add" = "Drupal\openquestions\Form\OqApplicationForm",
 *			 "edit" = "Drupal\openquestions\Form\OqApplicationForm",
 *			 "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *		 },
 *		 "route_provider" = {
 *			 "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *		 }
 *	 },
 *	 base_table = "oq_application",
 *	 data_table = "oq_application_field_data",
 *	 translatable = TRUE,
 *	 admin_permission = "administer oq application",
 *	 entity_keys = {
 *		 "id" = "id",
 *		 "langcode" = "langcode",
 *		 "label" = "label",
 *		 "uuid" = "uuid",
 *		 "owner" = "uid",
 *	 },
 *	 links = {
 *		 "collection" = "/admin/content/oq-application",
 *		 "add-form" = "/oq-application/add",
 *		 "canonical" = "/oq-application/{oq_application}",
 *		 "edit-form" = "/oq-application/{oq_application}/edit",
 *		 "delete-form" = "/oq-application/{oq_application}/delete",
 *	 },
 *	 field_ui_base_route = "entity.oq_application.settings",
 * )
 */
class OqApplication extends ContentEntityBase implements OqApplicationInterface {

	use EntityChangedTrait;
	use EntityOwnerTrait;

	/**
	 * {@inheritdoc}
	 */
	public function preSave(EntityStorageInterface $storage) {
		parent::preSave($storage);
		if (!$this->getOwnerId()) {
			// If no owner has been set explicitly, make the anonymous user the owner.
			$this->setOwnerId(0);
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
		$fields = parent::baseFieldDefinitions($entity_type);

		$fields['label'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Label'))
			->setRequired(TRUE)
			->setSetting('max_length', 255)
			->setDisplayOptions('form', [
				'type' => 'string_textfield',
				'weight' => -20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'string',
				'weight' => -20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Status'))
			->setDefaultValue(TRUE)
			->setSetting('on_label', 'Enabled')
			->setDisplayOptions('form', [
				'type' => 'boolean_checkbox',
				'settings' => [
					'display_label' => FALSE,
				],
				'weight' => 0,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'boolean',
				'label' => 'above',
				'weight' => 0,
				'settings' => [
					'format' => 'enabled-disabled',
				],
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['description'] = BaseFieldDefinition::create('text_long')
			->setTranslatable(TRUE)
			->setLabel(t('Why should you be selected as a voter?'))
			->setDescription(t('List your qualifications, expertise, etc.'))
			->setRequired(TRUE)
			->setDisplayOptions('form', [
				'type' => 'text_textarea',
				'weight' => -10,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'text_default',
				'label' => 'above',
				'weight' => -10,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['links'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Links'))
			->setDescription(t('Enter your websites, social media accounts, etc.'))
			->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
			->setRequired(FALSE)
			->setSetting('max_length', 255)
			->setDisplayOptions('form', [
				'type' => 'string_textfield',
				'weight' => -5,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'string',
				'weight' => -5,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['uid'] = BaseFieldDefinition::create('entity_reference')
			->setTranslatable(TRUE)
			->setLabel(t('Author'))
			->setSetting('target_type', 'user')
			->setDefaultValueCallback(static::class . '::getDefaultEntityOwner')
			->setDisplayOptions('form', [
				'type' => 'entity_reference_autocomplete',
				'settings' => [
					'match_operator' => 'CONTAINS',
					'size' => 60,
					'placeholder' => '',
				],
				'weight' => 15,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'author',
				'weight' => 15,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['created'] = BaseFieldDefinition::create('created')
			->setLabel(t('Authored on'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq application was created.'))
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('form', [
				'type' => 'datetime_timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['changed'] = BaseFieldDefinition::create('changed')
			->setLabel(t('Changed'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq application was last edited.'));

		$fields['disposition'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Disposition'))
			->setDescription(t('The current disposition of this application.'))
			->setRequired(TRUE)
			->setSetting('target_type', 'taxonomy_term')
			->setSetting('handler', 'default:taxonomy_term')
			->setSetting('handler_settings', [
				'target_bundles' => [
					'oq_application_disposition' => 'oq_application_disposition',
				],
			])
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		$fields['histories'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Histories'))
			->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
			->setDescription(t('Disposition actions that were taken on this application.'))
			->setSetting('target_type', 'oq_application_history')
			->setSetting('handler', 'default:oq_application_history')
			->setSetting('handler_settings', [
				'target_bundles' => NULL,
				'auto_create' => TRUE,
				'sort' => [
					'field' => 'created',
					'direction' => 'DESC',
				],
			])
			->setDisplayOptions('view', [
				'label'	=> 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		return $fields;
	}

}
