<?php

namespace Drupal\openquestions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\openquestions\Model\OqApplicationHistoryInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the oq application history entity class.
 *
 * @ContentEntityType(
 *	 id = "oq_application_history",
 *	 label = @Translation("OQ Application History"),
 *	 label_collection = @Translation("OQ Application Histories"),
 *	 label_singular = @Translation("oq application history"),
 *	 label_plural = @Translation("oq application histories"),
 *	 label_count = @PluralTranslation(
 *		 singular = "@count oq application histories",
 *		 plural = "@count oq application histories",
 *	 ),
 *	 handlers = {
 *		 "list_builder" = "Drupal\openquestions\List\OqApplicationHistoryListBuilder",
 *		 "views_data" = "Drupal\views\EntityViewsData",
 *		 "access" = "Drupal\openquestions\Access\OqApplicationHistoryAccessControlHandler",
 *		 "form" = {
 *			 "add" = "Drupal\openquestions\Form\OqApplicationHistoryForm",
 *			 "edit" = "Drupal\openquestions\Form\OqApplicationHistoryForm",
 *			 "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *		 },
 *		 "route_provider" = {
 *			 "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *		 }
 *	 },
 *	 base_table = "oq_application_history",
 *	 data_table = "oq_application_history_field_data",
 *	 translatable = TRUE,
 *	 admin_permission = "administer oq application history",
 *	 entity_keys = {
 *		 "id" = "id",
 *		 "langcode" = "langcode",
 *		 "label" = "label",
 *		 "uuid" = "uuid",
 *		 "owner" = "uid",
 *	 },
 *	 links = {
 *		 "collection" = "/admin/content/oq-application-history",
 *		 "add-form" = "/oq-application-history/add",
 *		 "canonical" = "/oq-application-history/{oq_application_history}",
 *		 "edit-form" = "/oq-application-history/{oq_application_history}/edit",
 *		 "delete-form" = "/oq-application-history/{oq_application_history}/delete",
 *	 },
 *	 field_ui_base_route = "entity.oq_application_history.settings",
 * )
 */
class OqApplicationHistory extends ContentEntityBase implements OqApplicationHistoryInterface {

	use EntityChangedTrait;
	use EntityOwnerTrait;

	/**
	 * {@inheritdoc}
	 */
	public function preSave(EntityStorageInterface $storage) {
		parent::preSave($storage);
		if (!$this->getOwnerId()) {
			// If no owner has been set explicitly, make the anonymous user the owner.
			$this->setOwnerId(0);
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

		$fields = parent::baseFieldDefinitions($entity_type);

		$fields['label'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Label'))
			->setRequired(TRUE)
			->setSetting('max_length', 255)
			->setDisplayOptions('form', [
				'type' => 'string_textfield',
				'weight' => -5,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'string',
				'weight' => -5,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Status'))
			->setDefaultValue(TRUE)
			->setSetting('on_label', 'Enabled')
			->setDisplayOptions('form', [
				'type' => 'boolean_checkbox',
				'settings' => [
					'display_label' => FALSE,
				],
				'weight' => 0,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'boolean',
				'label' => 'above',
				'weight' => 0,
				'settings' => [
					'format' => 'enabled-disabled',
				],
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['description'] = BaseFieldDefinition::create('text_long')
			->setTranslatable(TRUE)
			->setLabel(t('Description'))
			->setDescription(t('An explanation of why the disposition of the containing application was changed.'))
			->setRequired(TRUE)
			->setDisplayOptions('form', [
				'type' => 'text_textarea',
				'weight' => 10,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'text_default',
				'label' => 'above',
				'weight' => 10,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['uid'] = BaseFieldDefinition::create('entity_reference')
			->setTranslatable(TRUE)
			->setLabel(t('Author'))
			->setSetting('target_type', 'user')
			->setDefaultValueCallback(static::class . '::getDefaultEntityOwner')
			->setDisplayOptions('form', [
				'type' => 'entity_reference_autocomplete',
				'settings' => [
					'match_operator' => 'CONTAINS',
					'size' => 60,
					'placeholder' => '',
				],
				'weight' => 15,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'author',
				'weight' => 15,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['created'] = BaseFieldDefinition::create('created')
			->setLabel(t('Authored on'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq application history was created.'))
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('form', [
				'type' => 'datetime_timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['changed'] = BaseFieldDefinition::create('changed')
			->setLabel(t('Changed'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq application history was last edited.'));

		$fields['disposition_before'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Disposition before'))
			->setDescription(t('The disposition the containing application had before it was changed.'))
			->setRequired(TRUE)
			->setSetting('target_type', 'taxonomy_term')
			->setSetting('handler', 'default:taxonomy_term')
			->setSetting('handler_settings', [
				'target_bundles' => [
					'oq_application_disposition' => 'oq_application_disposition',
				],
			])
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'author',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		$fields['disposition_after'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Disposition after'))
			->setDescription(t('The disposition the containing application had after it was changed.'))
			->setRequired(TRUE)
			->setSetting('target_type', 'taxonomy_term')
			->setSetting('handler', 'default:taxonomy_term')
			->setSetting('handler_settings', [
				'target_bundles' => [
					'oq_application_disposition' => 'oq_application_disposition',
				],
			])
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		return $fields;
	}

}
