<?php

namespace Drupal\openquestions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\openquestions\Model\OqItemInterface;
use Drupal\openquestions\Service\VoteDataFactoryInterface;
use Drupal\openquestions\Model\VoteDataInterface;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the oq item entity class.
 *
 * @ContentEntityType(
 *	 id = "oq_item",
 *	 label = @Translation("OQ Item"),
 *	 label_collection = @Translation("OQ Items"),
 *	 label_singular = @Translation("oq item"),
 *	 label_plural = @Translation("oq items"),
 *	 label_count = @PluralTranslation(
 *		 singular = "@count oq items",
 *		 plural = "@count oq items",
 *	 ),
 *	 bundle_label = @Translation("OQ Item type"),
 *	 handlers = {
 *		 "view_builder" = "Drupal\openquestions\Display\OqItemViewBuilder",
 *		 "list_builder" = "Drupal\openquestions\List\OqItemListBuilder",
 *		 "views_data" = "Drupal\views\EntityViewsData",
 *		 "access" = "Drupal\openquestions\Access\OqItemAccessControlHandler",
 *		 "form" = {
 *			 "add" = "Drupal\openquestions\Form\OqItemForm",
 *			 "edit" = "Drupal\openquestions\Form\OqItemForm",
 *			 "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *		 },
 *		 "route_provider" = {
 *			 "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *		 }
 *	 },
 *	 base_table = "oq_item",
 *	 data_table = "oq_item_field_data",
 *	 translatable = TRUE,
 *	 admin_permission = "administer oq item types",
 *	 entity_keys = {
 *		 "id" = "id",
 *		 "langcode" = "langcode",
 *		 "bundle" = "bundle",
 *		 "label" = "label",
 *		 "uuid" = "uuid",
 *		 "owner" = "uid",
 *	 },
 *	 links = {
 *		 "collection" = "/admin/content/oq-item",
 *		 "add-form" = "/oq-item/add/{oq_item_type}",
 *		 "add-page" = "/oq-item/add",
 *		 "canonical" = "/oq-item/{oq_item}",
 *		 "edit-form" = "/oq-item/{oq_item}/edit",
 *		 "delete-form" = "/oq-item/{oq_item}/delete",
 *	 },
 *	 bundle_entity_type = "oq_item_type",
 *	 field_ui_base_route = "entity.oq_item_type.edit_form",
 * )
 */
class OqItem extends ContentEntityBase implements OqItemInterface {

	use EntityChangedTrait;
	use EntityOwnerTrait;

	/**
	 * The vote data factory.
	 *
	 * @var \Drupal\openquestions\Service\VoteDataFactoryInterface
	 */
	protected $voteDataFactory;

	/**
	 * Our logger.
	 *
	 * @var \Drupal\Core\Logger\LoggerChannelInterface
	 */
	protected $logger;

	/**
	 * Overriden to get our services.
	 */
	public function __construct(array $values, $entity_type, $bundle = FALSE, $translations = []) {
		parent::__construct($values, $entity_type, $bundle, $translations);

		//	"Provide a method of DI for entities", https://www.drupal.org/project/drupal/issues/2142515
		$this->voteDataFactory = \Drupal::service('openquestions.oq_vote_data_factory');
		$this->logger = \Drupal::service('openquestions.logger.channel.openquestions');
	}

	/**
	 * {@inheritdoc}
	 */
	public function getVoteData() : VoteDataInterface {
		return $this->voteDataFactory->createVoteDataForItem($this->id());
	}

	/**
	 * {@inheritdoc}
	 */
	public function getQuestions() {
		return $this->questions->referencedEntities();
	}

	/**
	 * {@inheritdoc}
	 */
	public function preSave(EntityStorageInterface $storage) {
		parent::preSave($storage);
		if (!$this->getOwnerId()) {
			// If no owner has been set explicitly, make the anonymous user the owner.
			$this->setOwnerId(0);
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

		$fields = parent::baseFieldDefinitions($entity_type);

		$fields['label'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Label'))
			->setRequired(TRUE)
			->setSetting('max_length', 255)
			->setDisplayOptions('form', [
				'type' => 'string_textfield',
				'weight' => -5,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'string',
				'weight' => -5,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Status'))
			->setDefaultValue(TRUE)
			->setSetting('on_label', 'Enabled')
			->setDisplayOptions('form', [
				'type' => 'boolean_checkbox',
				'settings' => [
					'display_label' => FALSE,
				],
				'weight' => 0,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'boolean',
				'label' => 'above',
				'weight' => 0,
				'settings' => [
					'format' => 'enabled-disabled',
				],
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['description'] = BaseFieldDefinition::create('text_long')
			->setTranslatable(TRUE)
			->setLabel(t('Description'))
			->setDisplayOptions('form', [
				'type' => 'text_textarea',
				'weight' => 10,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'text_default',
				'label' => 'above',
				'weight' => 10,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['uid'] = BaseFieldDefinition::create('entity_reference')
			->setTranslatable(TRUE)
			->setLabel(t('Author'))
			->setSetting('target_type', 'user')
			->setDefaultValueCallback(static::class . '::getDefaultEntityOwner')
			->setDisplayOptions('form', [
				'type' => 'entity_reference_autocomplete',
				'settings' => [
					'match_operator' => 'CONTAINS',
					'size' => 60,
					'placeholder' => '',
				],
				'weight' => 15,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'author',
				'weight' => 15,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['created'] = BaseFieldDefinition::create('created')
			->setLabel(t('Authored on'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq item was created.'))
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('form', [
				'type' => 'datetime_timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['changed'] = BaseFieldDefinition::create('changed')
			->setLabel(t('Changed'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq item was last edited.'));

		$fields['questions'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Questions'))
			->setCardinality(FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED)
			->setDescription(t('The questions.'))
			->setSetting('target_type', 'oq_question')
			->setSetting('handler', 'default:oq_question')
			->setSetting('handler_settings', [
				'target_bundles' => NULL,
				'auto_create' => TRUE,
				'sort' => [
					'field' => 'changed',
					'direction' => 'DESC',
				],
			])
			->setDisplayOptions('view', [
				'label'	=> 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		$fields['locked_status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Locked'))
			->setDefaultValue(FALSE);

		$fields['locked_date'] = BaseFieldDefinition::create('datetime')
			->setLabel(t('Locked date'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq question was locked.'));

		$fields['locked_reason'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Locked reason'))
			->setSetting('max_length', 255);

		return $fields;
	}

}
