<?php

namespace Drupal\openquestions\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;
use Drupal\openquestions\Model\OqVoteInterface;
use Drupal\openquestions\Event\OQVoteEvent;
use Drupal\user\EntityOwnerTrait;

/**
 * Defines the oq vote entity class.
 *
 * @ContentEntityType(
 *	 id = "oq_vote",
 *	 label = @Translation("OQ Vote"),
 *	 label_collection = @Translation("OQ Votes"),
 *	 label_singular = @Translation("oq vote"),
 *	 label_plural = @Translation("oq votes"),
 *	 label_count = @PluralTranslation(
 *		 singular = "@count oq votes",
 *		 plural = "@count oq votes",
 *	 ),
 *	 handlers = {
 *		 "view_builder" = "Drupal\openquestions\Display\OqVoteViewBuilder",
 *		 "list_builder" = "Drupal\openquestions\List\OqVoteListBuilder",
 *		 "views_data" = "Drupal\views\EntityViewsData",
 *		 "access" = "Drupal\openquestions\Access\OqVoteAccessControlHandler",
 *		 "form" = {
 *			 "add" = "Drupal\openquestions\Form\OqVoteForm",
 *			 "edit" = "Drupal\openquestions\Form\OqVoteForm",
 *			 "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *		 },
 *		 "route_provider" = {
 *			 "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *		 }
 *	 },
 *	 base_table = "oq_vote",
 *	 data_table = "oq_vote_field_data",
 *	 translatable = TRUE,
 *	 admin_permission = "administer oq vote",
 *	 entity_keys = {
 *		 "id" = "id",
 *		 "langcode" = "langcode",
 *		 "label" = "label",
 *		 "uuid" = "uuid",
 *		 "owner" = "uid",
 *	 },
 *	 links = {
 *		 "collection" = "/admin/content/oq-vote",
 *		 "add-form" = "/oq-vote/add",
 *		 "canonical" = "/oq-vote/{oq_vote}",
 *		 "edit-form" = "/oq-vote/{oq_vote}/edit",
 *		 "delete-form" = "/oq-vote/{oq_vote}/delete",
 *	 },
 *	 field_ui_base_route = "entity.oq_vote.settings",
 * )
 */
class OqVote extends ContentEntityBase implements OqVoteInterface {
	use EntityChangedTrait;
	use EntityOwnerTrait;

	/**
	 * Used to dispatch an event when this is created.
	 *
	 * @var \Symfony\Contracts\EventDispatcher\EventDispatcherInterface
	 */
	protected $eventDispatcher;

	/**
	 * Our logger.
	 *
	 * @var \Drupal\Core\Logger\LoggerChannelInterface
	 */
	protected $logger;

	/**
	 * Overriden to get our services.
	 */
	public function __construct(array $values, $entity_type, $bundle = FALSE, $translations = []) {
		parent::__construct($values, $entity_type, $bundle, $translations);

		//	"Provide a method of DI for entities", https://www.drupal.org/project/drupal/issues/2142515
		$this->eventDispatcher = \Drupal::service('event_dispatcher');
		$this->logger = \Drupal::service('openquestions.logger.channel.openquestions');
	}

	/**
	 * Saves if this was a new vote.
	 *
	 * @var bool
	 */
	protected $wasNew;

	/**
	 * {@inheritdoc}
	 */
	public function preSave(EntityStorageInterface $storage) {
		$this->wasNew = $this->id() ? FALSE : TRUE;

		parent::preSave($storage);
		if (!$this->getOwnerId()) {
			// If no owner has been set explicitly, make the anonymous user the owner.
			$this->setOwnerId(0);
		}
	}

	/**
	 * {@inheritdoc}
	 */
	public function postSave(EntityStorageInterface $storage, $update = TRUE) {
		if ($this->wasNew) {
			$event = new OQVoteEvent($this);
			$event = $this->eventDispatcher->dispatch($event, OQVoteEvent::EVENT);
		}

		$this->wasNew = FALSE;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

		$fields = parent::baseFieldDefinitions($entity_type);

		$fields['label'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Label'))
			->setRequired(TRUE)
			->setSetting('max_length', 255)
			->setDisplayOptions('form', [
				'type' => 'string_textfield',
				'weight' => -5,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'hidden',
				'type' => 'string',
				'weight' => -5,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Status'))
			->setDefaultValue(TRUE)
			->setSetting('on_label', 'Enabled')
			->setDisplayOptions('form', [
				'type' => 'boolean_checkbox',
				'settings' => [
					'display_label' => FALSE,
				],
				'weight' => 0,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'boolean',
				'label' => 'above',
				'weight' => 0,
				'settings' => [
					'format' => 'enabled-disabled',
				],
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['description'] = BaseFieldDefinition::create('text_long')
			->setTranslatable(TRUE)
			->setLabel(t('Description'))
			->setDisplayOptions('form', [
				'type' => 'text_textarea',
				'weight' => 10,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'type' => 'text_default',
				'label' => 'above',
				'weight' => 10,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['uid'] = BaseFieldDefinition::create('entity_reference')
			->setTranslatable(TRUE)
			->setLabel(t('Author'))
			->setSetting('target_type', 'user')
			->setDefaultValueCallback(static::class . '::getDefaultEntityOwner')
			->setDisplayOptions('form', [
				'type' => 'entity_reference_autocomplete',
				'settings' => [
					'match_operator' => 'CONTAINS',
					'size' => 60,
					'placeholder' => '',
				],
				'weight' => 15,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'author',
				'weight' => 15,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['created'] = BaseFieldDefinition::create('created')
			->setLabel(t('Authored on'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq vote was created.'))
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('form', [
				'type' => 'datetime_timestamp',
				'weight' => 20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['changed'] = BaseFieldDefinition::create('changed')
			->setLabel(t('Changed'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq vote was last edited.'));

		$fields['vote_value'] = BaseFieldDefinition::create('integer')
			->setLabel(t('Vote value'))
			->setTranslatable(TRUE)
			->setDescription(t('The selected value from the oq_question.'))
			->setDisplayOptions('view', [
				'label' => 'above',
				'type' => 'number_unformatted',
				'weight' => 20,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayOptions('form', [
				'type' => 'number',
				'weight' => 20,
			])
			->setDisplayConfigurable('view', TRUE);

		$fields['item'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Item'))
			->setCardinality(1)
			->setDescription(t('The item being voted on.'))
			->setSetting('target_type', 'oq_item')
			->setSetting('handler', 'default:oq_item')
			->setSetting('handler_settings', [
				'target_bundles' => NULL,
				'auto_create' => TRUE,
				'sort' => [
					'field' => 'changed',
					'direction' => 'DESC',
				],
			])
			->setDisplayOptions('view', [
				'label'	=> 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		$fields['question'] = BaseFieldDefinition::create('entity_reference')
			->setLabel(t('Question'))
			->setCardinality(1)
			->setDescription(t('The question being voted on.'))
			->setSetting('target_type', 'oq_question')
			->setSetting('handler', 'default:oq_question')
			->setSetting('handler_settings', [
				'target_bundles' => NULL,
				'auto_create' => TRUE,
				'sort' => [
					'field' => 'changed',
					'direction' => 'DESC',
				],
			])
			->setDisplayOptions('view', [
				'label'	=> 'hidden',
				'type' => 'entity_reference_label',
				'weight' => 0,
			])
			->setDisplayOptions('form', [
				'type' => 'options_select',
				'weight' => -2,
			])
			->setDisplayConfigurable('form', TRUE)
			->setDisplayConfigurable('view', TRUE);

		$fields['locked_status'] = BaseFieldDefinition::create('boolean')
			->setLabel(t('Locked'))
			->setDefaultValue(FALSE);

		$fields['locked_date'] = BaseFieldDefinition::create('datetime')
			->setLabel(t('Locked date'))
			->setTranslatable(TRUE)
			->setDescription(t('The time that the oq question was locked.'));

		$fields['locked_reason'] = BaseFieldDefinition::create('string')
			->setTranslatable(TRUE)
			->setLabel(t('Locked reason'))
			->setSetting('max_length', 255);

		return $fields;
	}

}
